@@ -40,7 +40,7 @@ static char  *license_msg[] = {
  */
 
 #ifndef lint
-static char rcsid[] = "$Id: gzip.c,v 0.17 1993/03/18 18:14:56 jloup Exp $";
+static char rcsid[] = "$Id: gzip.c,v 0.19 1993/06/01 14:21:46 jloup Exp $";
 #endif
 
 #include <stdio.h>
@@ -175,6 +175,7 @@ DECLARE(uch, window, 2L*WSIZE);
 
 		/* local variables */
 
+int ascii = 0;        /* convert end-of-lines to local OS conventions */
 int to_stdout = 0;    /* output to stdout (-c) */
 int decompress = 0;   /* decompress (-d) */
 int force = 0;        /* don't ask questions, compress links (-f) */
@@ -214,7 +215,7 @@ unsigned outcnt;           /* bytes in output buffer */
 struct option longopts[] =
 {
  /* { name  has_arg  *flag  val } */
- /* {"ascii",      0, 0, 'a'},  ascii text mode */
+    {"ascii",      0, 0, 'a'}, /* ascii text mode */
     {"to-stdout",  0, 0, 'c'}, /* write output on standard output */
     {"stdout",     0, 0, 'c'}, /* write output on standard output */
     {"decompress", 0, 0, 'd'}, /* decompress */
@@ -269,27 +270,28 @@ int (*work) OF((int infile, int outfile)) = zip; /* function to call */
 /* ======================================================================== */
 local void usage()
 {
-    fprintf(stderr,
-#ifdef LZW
-#  ifdef NO_DIR
-            "usage: %s [-cdfhLtvVZ19] [-b maxbits] [file ...]\n",
-#  else
-            "usage: %s [-cdfhLrtvVZ19] [-b maxbits] [file ...]\n",
-#  endif
-#else /* !LZW */
-#  ifdef NO_DIR
-            "usage: %s [-cdfhLStvV19] [file ...]\n",
-#  else
-            "usage: %s [-cdfhLrStvV19] [file ...]\n",
-#  endif
-#endif /* LZW */
-             progname);
+    fprintf(stderr, "usage: %s [-%scdfhL%stvV19] [-S suffix] [file ...]\n",
+	    progname,
+#if O_BINARY
+	    "a",
+#else
+	    "",
+#endif
+#ifdef NO_DIR
+	    ""
+#else
+	    "r"
+#endif
+	    );
 }
+
 /* ======================================================================== */
 local void help()
 {
     static char  *help_msg[] = {
-/* -a --ascii       ascii text; convert end-of-lines to local OS conventions */
+#if O_BINARY
+ " -a --ascii       ascii text; convert end-of-lines using local conventions",
+#endif
  " -c --stdout      write on standard output, keep original files unchanged",
  " -d --decompress  decompress",
 /* -e --encrypt     encrypt */
@@ -433,9 +435,11 @@ int main (argc, argv)
     strncpy(z_suffix, Z_SUFFIX, sizeof(z_suffix)-1);
     z_len = strlen(z_suffix);
 
-    while ((optc = getopt_long (argc, argv, "b:cdfhLqrS:tvVZ123456789",
+    while ((optc = getopt_long (argc, argv, "ab:cdfhLqrS:tvVZ123456789",
 				longopts, (int *)0)) != EOF) {
 	switch (optc) {
+        case 'a':
+            ascii = 1; break;
 	case 'b':
 	    maxbits = atoi(optarg);
 	    break;
@@ -500,6 +504,13 @@ int main (argc, argv)
 
     file_count = argc - optind;
 
+#if O_BINARY
+#else
+    if (ascii && !quiet) {
+	fprintf(stderr, "%s: option --ascii ignored on this system\n",
+		progname);
+    }
+#endif
     if (do_lzw && !decompress) work = lzw;
 
     /* Allocate all global buffers (for DYN_ALLOC option) */
@@ -516,7 +527,7 @@ int main (argc, argv)
 
     /* And get to work */
     if (file_count != 0) {
-	if (to_stdout && !test) {
+	if (to_stdout && !test && (!decompress || !ascii)) {
 	    SET_BINARY_MODE(fileno(stdout));
 	}
         while (optind < argc) {
@@ -555,9 +566,12 @@ local void treat_stdin()
 	do_exit(ERROR);
     }
 
-    SET_BINARY_MODE(fileno(stdin));
-    if (!test) SET_BINARY_MODE(fileno(stdout));
-
+    if (decompress || !ascii) {
+	SET_BINARY_MODE(fileno(stdin));
+    }
+    if (!test && (!decompress || !ascii)) {
+	SET_BINARY_MODE(fileno(stdout));
+    }
     strcpy(ifname, "stdin");
     strcpy(ofname, "stdout");
 
@@ -668,7 +682,8 @@ local void treat_file(iname)
      * parameter is ignored but required by some systems (VMS) and forbidden
      * on other systems (MacOS).
      */
-    ifd = OPEN(ifname, O_RDONLY | O_BINARY, RW_USER);
+    ifd = OPEN(ifname, ascii && !decompress ? O_RDONLY : O_RDONLY | O_BINARY,
+	       RW_USER);
     if (ifd == -1) {
 	fprintf(stderr, "%s: ", progname);
 	perror(ifname);
@@ -761,7 +776,11 @@ local int create_outfile()
 {
     struct stat	ostat; /* stat for ofname */
     int len;
+    int flags = O_WRONLY | O_CREAT | O_EXCL | O_BINARY;
 
+    if (ascii && decompress) {
+	flags &= ~O_BINARY; /* force ascii text mode */
+    }
     for (;;) {
         len = strlen(ofname);
         if (len == 0 || ofname[len] == PATH_SEP) break;
@@ -771,7 +790,7 @@ local int create_outfile()
 	}
 	/* Create the output file */
 	remove_ofname = 1;
-	ofd = OPEN(ofname, O_WRONLY|O_CREAT|O_EXCL|O_BINARY, RW_USER);
+	ofd = OPEN(ofname, flags, RW_USER);
 	if (ofd == -1) {
 	    perror(ofname);
 	    close(ifd);
